@@ -1,25 +1,29 @@
 ################################################################################
-#' Data-driven Bandwidth Selection for Local Polynomial Density Estimation
+#' @title Data-driven Bandwidth Selection for Local Polynomial Density Estimation
 #'
-#' \code{lpbwdensity} implements the bandwidth selector for local polynomial based density (and derivatives) estimation,
-#'   proposed in Cattaneo, Jansson and Ma (2017).
+#' @description \code{lpbwdensity} implements the bandwidth selector for local polynomial
+#'   based density (and derivatives) estimation, proposed in Cattaneo, Jansson and Ma (2017a).
+#'   See Cattaneo, Jansson and Ma (2017b) for more implementation details and illustrations.
 #'
-#' Companion command: \code{\link{lpdensity}} for local polynomial density estimation.
+#'   Companion command: \code{\link{lpdensity}} for local polynomial density estimation.
+#'
+#'   For more details, and related Stata and R packages useful for empirical analysis,
+#'   visit \url{https://sites.google.com/site/nppackages/}.
 #'
 #' @param data Numeric vector or one dimensional matrix / data frame, the raw data.
 #' @param grid Numeric vector or one dimensional matrix / data frame, the grid on which
 #'   density is estimated. When set to default, grid points will be chosen as 0.05-0.95
 #'   percentiles of the data, with 0.05 step size.
 #' @param bwselect String, the method for data-driven bandwidth selection. This option will be
-#'   ignored if \code{bw} is provided. Can be (1) \code{"MSE"} (default, mean squared error-optimal
-#'   bandwidth selected for each grid point); or (2) \code{"IMSE"} (integrated MSE-optimal bandwidth,
-#'   common for all grid points); (3) \code{"ROT"} (rule-of-thumb bandwidth with Gaussian
-#'   reference model); and (4) \code{"IROT"} (integrated rule-of-thumb bandwidth with Gaussian
+#'   ignored if \code{bw} is provided. Can be (1) \code{"mse-dpi"} (default, mean squared error-optimal
+#'   bandwidth selected for each grid point); or (2) \code{"imse-dpi"} (integrated MSE-optimal bandwidth,
+#'   common for all grid points); (3) \code{"mse-rot"} (rule-of-thumb bandwidth with Gaussian
+#'   reference model); and (4) \code{"imse-rot"} (integrated rule-of-thumb bandwidth with Gaussian
 #'   reference model).
-#' @param p Integer, the order of the local-polynomial used to construct point
-#'   estimates. Should be larger than 0. (Default is 2.)
-#' @param v Integer, the derivative to be estimated. Default is \code{1}, which is the density function.
-#'   Should be nonnegative.
+#' @param p Integer, nonnegative, the order of the local-polynomial used to construct point
+#'   estimates. (Default is 2.)
+#' @param v Integer, nonnegative, the derivative of distribution function to be estimated. \code{0} for
+#'   the distribution function, \code{1} (default) for the density funtion, etc.
 #' @param kernel String, the kernel function, should be one of \code{"triangular"}, \code{"uniform"} or
 #'   \code{"epanechnikov"}.
 #' @param Cweights Numeric vector or one dimensional matrix / data frame, the weights used
@@ -38,7 +42,9 @@
 #' \item{opt}{A list containing options passed to the function.}
 #'
 #' @references
-#' M. D. Cattaneo, M. Jansson and X. Ma. (2017). \href{http://www-personal.umich.edu/~cattaneo/papers/Cattaneo-Jansson-Ma_2017_LocPolDensity.pdf}{Simple Local Regression Distribution Estimators}. Working Paper, University of Michigan.
+#' M. D. Cattaneo, M. Jansson and X. Ma. (2017a). \href{http://www-personal.umich.edu/~cattaneo/papers/Cattaneo-Jansson-Ma_2017_LocPolDensity.pdf}{Simple Local Regression Distribution Estimators}. Working Paper, University of Michigan.
+#'
+#' M. D. Cattaneo, M. Jansson and X. Ma. (2017b). \href{http://www-personal.umich.edu/~cattaneo/papers/Cattaneo-Jansson-Ma_2017_lpdensity.pdf}{\code{lpdensity}: Local Polynomial Density Estimation and Inference}. Working Paper, University of Michigan.
 #'
 #' @author
 #' Matias D. Cattaneo, University of Michigan. \email{cattaneo@umich.edu}.
@@ -51,13 +57,13 @@
 #'
 #' @examples
 #' set.seed(42); X <- rnorm(1000)
-#' lpbwdensity(X)
+#' summary(lpbwdensity(X))
 #'
 #' @export
-lpbwdensity <- function(data, grid=c(), bwselect=c("MSE", "IMSE", "ROT", "IROT"),
-                        p=c(), v=c(),
+lpbwdensity <- function(data, grid=NULL, bwselect=c("mse-dpi", "imse-dpi", "mse-rot", "imse-rot"),
+                        p=NULL, v=NULL,
                         kernel=c("triangular", "uniform", "epanechnikov"),
-                        Cweights=c(), Pweights=c(), regularize=TRUE) {
+                        Cweights=NULL, Pweights=NULL, regularize=TRUE) {
   ################################################################################
   # Input Error Handling
   ################################################################################
@@ -88,11 +94,17 @@ lpbwdensity <- function(data, grid=c(), bwselect=c("MSE", "IMSE", "ROT", "IROT")
 
   # bwselect
   if (length(bwselect) == 0) {
-    bwselect <- "MSE"
+    bwselect <- "mse-dpi"
   } else {
-    bwselect <- toupper(bwselect[1])
+    bwselect <- tolower(bwselect[1])
   }
-  if (!bwselect%in%c("MSE", "IMSE", "ROT", "IROT")) stop("Incorrect bandwidth selection method specified.\n")
+  ### BACKWARD COMPATIBILITY FOR VERSION 0.1
+  if (bwselect == "mse" ) { bwselect <- "mse-dpi" }
+  if (bwselect == "imse") { bwselect <- "imse-dpi" }
+  if (bwselect == "rot" ) { bwselect <- "mse-rot" }
+  if (bwselect == "irot") { bwselect <- "imse-rot" }
+  ### END BACKWARD COMPATIBILITY FOR VERSION 0.1
+  if (!bwselect%in%c("mse-dpi", "imse-dpi", "mse-rot", "imse-rot")) stop("Incorrect bandwidth selection method specified.\n")
 
   # p
   if (length(p) == 0) {
@@ -107,7 +119,7 @@ lpbwdensity <- function(data, grid=c(), bwselect=c("MSE", "IMSE", "ROT", "IROT")
   # v
   if (length(v) == 0) {
     flag_no_v <- TRUE
-    v <- 1
+    v <- min(p, 1)
   } else if ((length(v) > 1) | !(v[1]%in%c(0:20)) | (v[1]>p)) {
     stop("Derivative order v incorrectly specified.\n")
   } else {
@@ -182,11 +194,11 @@ lpbwdensity <- function(data, grid=c(), bwselect=c("MSE", "IMSE", "ROT", "IROT")
   # Bandwidth Estimation
   ################################################################################
 
-  if (bwselect == "MSE") {
+  if (bwselect == "mse-dpi") {
     bw <- bw_MSE(data=data, grid=grid, p=p, v=v, kernel=kernel, Cweights=Cweights, Pweights=Pweights, regularize=regularize)
-  } else if (bwselect == "IMSE") {
+  } else if (bwselect == "imse-dpi") {
     bw <- bw_IMSE(data=data, grid=grid, p=p, v=v, kernel=kernel, Cweights=Cweights, Pweights=Pweights, regularize=regularize)
-  } else if (bwselect == "ROT") {
+  } else if (bwselect == "mse-rot") {
     bw <- bw_ROT(data=data, grid=grid, p=p, v=v, kernel=kernel, regularize=regularize)
   } else {
     bw <- bw_IROT(data=data, grid=grid, p=p, v=v, kernel=kernel, regularize=regularize)
@@ -229,14 +241,13 @@ print.CJMlpbwdensity <- function(x, ...) {
 
   cat("Call: lpbwdensity\n\n")
 
-  cat(paste("Sample size: ", x$opt$n,  ". Min. observation: ", round(x$opt$data_min, 3), ", Max. observation: ", round(x$opt$data_max, 3), ".\n",   sep=""))
-  cat(paste("Grid points: ", x$opt$ng, ". Min. grid: ",        round(x$opt$grid_min, 3), ", Max. grid: ",        round(x$opt$grid_max, 3), ".\n\n", sep=""))
-
-  cat(paste("Polynomial order for point estimation: ", x$opt$p, ".\n", sep=""))
-  cat(paste("Order of derivative estimated: ", x$opt$v, ".\n", sep=""))
-  cat(paste("Kernel function: ", x$opt$kernel, ".\n", sep=""))
-  cat(paste("Bandwidths estimated by: ", x$opt$bwselect, ".\n", sep=""))
+  cat(paste("Sample size                                   ", x$opt$n,        "\n", sep=""))
+  cat(paste("Polynomial order for point estimation (p=)    ", x$opt$p,        "\n", sep=""))
+  cat(paste("Order of derivative estimated         (v=)    ", x$opt$v,        "\n", sep=""))
+  cat(paste("Kernel function                               ", x$opt$kernel,   "\n", sep=""))
+  cat(paste("Bandwidth method                              ", x$opt$bwselect, "\n", sep=""))
   cat("\n")
+
   cat("Use summary(...) to show bandwidths.\n")
 }
 
@@ -249,20 +260,47 @@ print.CJMlpbwdensity <- function(x, ...) {
 #' @export
 summary.CJMlpbwdensity <- function(object, ...) {
   x <- object
-  colnames(x$BW) <- c("      grid", "        bw", "        nh")
+  args <- list(...)
+  if (is.null(args[['sep']]))   { sep <- 5 } else { sep <- args[['sep']] }
 
   cat("Call: lpbwdensity\n\n")
 
-  cat(paste("Sample size: ", x$opt$n,  ". Min. observation: ", round(x$opt$data_min, 3), ", Max. observation: ", round(x$opt$data_max, 3), ".\n",   sep=""))
-  cat(paste("Grid points: ", x$opt$ng, ". Min. grid: ",        round(x$opt$grid_min, 3), ", Max. grid: ",        round(x$opt$grid_max, 3), ".\n\n", sep=""))
-
-  cat(paste("Polynomial order for point estimation: ", x$opt$p, ".\n", sep=""))
-  cat(paste("Order of derivative estimated: ", x$opt$v, ".\n", sep=""))
-  cat(paste("Kernel function: ", x$opt$kernel, ".\n", sep=""))
-  cat(paste("Bandwidths estimated by: ", x$opt$bwselect, ".\n", sep=""))
+  cat(paste("Sample size                           (n=)    ", x$opt$n,        "\n", sep=""))
+  cat(paste("Polynomial order for point estimation (p=)    ", x$opt$p,        "\n", sep=""))
+  if (x$opt$v == 0) {
+  cat(paste("Distribution function estimated       (v=)    ", x$opt$v,        "\n", sep=""))
+  } else if (x$opt$v == 1) {
+  cat(paste("Density function estimated            (v=)    ", x$opt$v,        "\n", sep=""))
+  } else {
+  cat(paste("Order of derivative estimated         (v=)    ", x$opt$v,        "\n", sep=""))
+  }
+  cat(paste("Kernel function                               ", x$opt$kernel,   "\n", sep=""))
+  cat(paste("Bandwidth selection method                    ", x$opt$bwselect, "\n", sep=""))
   cat("\n")
-  cat("Estimation result:\n")
-  print(round(x$BW, 3))
+
+  ### print output
+  cat(paste(rep("=", 14 + 10 + 8), collapse="")); cat("\n")
+
+  cat(format("grid"            , width=14, justify="right"))
+  cat(format("bw"              , width=10, justify="right"))
+  cat(format("Eff.n"           , width=8 , justify="right"))
+  cat("\n")
+
+  cat(paste(rep("=", 14 + 10 + 8), collapse="")); cat("\n")
+
+  for (j in 1:nrow(x$BW)) {
+    cat(format(toString(j), width=4))
+    cat(format(sprintf("%3.3f", x$BW[j, "grid"]), width=10, justify="right"))
+    cat(format(sprintf("%3.3f", x$BW[j, "bw"])  , width=10, justify="right"))
+    cat(format(sprintf("%3.0f", x$BW[j, "nh"])  , width=8 , justify="right"))
+    cat("\n")
+    if (is.numeric(sep)) if (sep > 0) if (j %% sep == 0) {
+      cat(paste(rep("-", 14 + 10 + 8), collapse="")); cat("\n")
+    }
+  }
+
+  cat(paste(rep("=", 14 + 10 + 8), collapse="")); cat("\n")
+
 }
 
 
